﻿; gdi+ ahk tutorial 8 written by tic (Tariq Porter)
; Requires Gdip.ahk either in your Lib folder as standard library or using #Include
;
; Tutorial to write text onto a gui

#SingleInstance Force
;#NoEnv
;SetBatchLines -1

#Include ../Class_Gdip.ahk

oGdip := new _GDIP()

OnExit("ExitFunc")

; Set the width and height we want as our drawing area, to draw everything in. This will be the dimensions of our bitmap
w := 300
h := 200

; Create a layered window (+E0x80000 : must be used for UpdateLayeredWindow to work!) that is always on top (+AlwaysOnTop), has no taskbar entry or caption
;AHK v1
;Gui, 1: -Caption +E0x80000 +LastFound +AlwaysOnTop +ToolWindow +OwnDialogs
;Gui, 1: Add, Edit, w%Width% h20 y300 vMeEdit
;Gui, 1: Show, NA
oGui := GuiCreate("-Caption +E0x80000 +LastFound +AlwaysOnTop +ToolWindow +OwnDialogs")
oGui.Add("Edit", format("w{1} h20 y300 vMeEdit", w), "")
oGui.Show("NA")

; Create a gdi bitmap with width and height of what we are going to draw into it. This is the entire drawing area for everything
oHBitmap := new GDIP_HBitmap(w,h)

; Get a pointer to the graphics of the bitmap, for use with drawing functions
oGraphics := new GDIP_Graphics(oHBitmap)

; Set the smoothing mode to antialias = 4 to make shapes appear smother (only used for vector drawing and filling)
oGraphics.GdipSetSmoothingMode(4)

; Create a partially transparent, black brush (ARGB = Transparency, red, green, blue) to draw a rounded rectangle with
oBrush := new GDIP_Brush(0xaa000000)
; Fill the graphics of the bitmap with a rounded rectangle using the brush created
; Filling the entire graphics - from coordinates (0, 0) the entire width and height
; The last parameter (20) is the radius of the circles used for the rounded corners
oGraphics.fillRoundedRectangle(oBrush.ptr, [0,0,w,h], 20)
; Delete the brush as it is no longer needed and wastes memory
oBrush := ""

; We can specify the font to use. Here we use Arial as most systems should have this installed
; oFont := new GDIP_Font("Arial")

; There are a lot of things to cover with the function Gdip_TextToGraphics

; The 1st parameter is the graphics we wish to use (our canvas)

; The 2nd parameter is the text we wish to write. It can include new lines `n

; The 3rd parameter, the options are where all the action takes place...
; You can write literal x and y coordinates such as x20 y50 which would place the text at that position in pixels
; or you can include the last 2 parameters (Width and Height of the Graphics we will use) and then you can use x10p
; which will place the text at 10% of the width and y30p which is 30% of the height
; The same percentage marker may be used for width and height also, so w80p makes the bounding box of the rectangle the text
; will be written to 80% of the width of the graphics. If either is missed (as I have missed height) then the height of the bounding
; box will be made to be the height of the graphics, so 100%

; Any of the following words may be used also: Regular,Bold,Italic,BoldItalic,Underline,Strikeout to perform their associated action

; To justify the text any of the following may be used: Near,Left,Centre,Center,Far,Right with different spelling of words for convenience

; The rendering hint (the quality of the antialiasing of the text) can be specified with r, whose values may be:
; SystemDefault = 0
; SingleBitPerPixelGridFit = 1
; SingleBitPerPixel = 2
; AntiAliasGridFit = 3
; AntiAlias = 4

; The size can simply be specified with s

; The colour and opacity can be specified for the text also by specifying the ARGB as demonstrated with other functions such as the brush
; So cffff0000 would make a fully opaque red brush, so it is: cARGB (the literal letter c, follwed by the ARGB)

; The 4th parameter is the name of the font you wish to use

; As mentioned previously, you don not need to specify the last 2 parameters, the width and height, unless
; you are planning on using the p option with the x,y,w,h to use the percentage
; opts := "x10 y30p w80p Centre cbbff0000 r4 s20 Underline Italic"
sText := "Tutorial 8`n`nThank you for trying this example"
; oGraphics.DrawText(sText, opts, "Arial", w, h)

oFont := new GDIP_Font("Arial", 18)
oStringFormat := new GDIP_StringFormat(0x4000)
oStringFormat.GdipSetStringFormatAlign(1)
oBrush := new GDIP_Brush(0xaaff0000)
aRect := [0, 0, 150, 400]
oGraphics.GdipDrawString(sText, oFont.ptr, oStringFormat.ptr, oBrush.ptr, aRect)
oStringFormat := ""
oBrush := ""
oFont := ""

oGraphics.UpdateLayeredWindow(oGui.hwnd, [(A_ScreenWidth-w)//2, (A_ScreenHeight-h)//2, w, h])

; By placing this OnMessage here. The function WM_LBUTTONDOWN will be called every time the user left clicks on the gui
OnMessage(0x201, "WM_LBUTTONDOWN")


oGraphics.SelectObject()
; Now the bitmap may be deleted
oHBitmap := ""
oGraphics := ""
Return

;#######################################################################

; This function is called every time the user clicks on the gui
; The PostMessage will act on the last found window (this being the gui that launched the subroutine, hence the last parameter not being needed)
WM_LBUTTONDOWN()
{
   PostMessage(0xA1, 2)
}

;#######################################################################

ExitFunc(ExitReason, ExitCode)
{
   global
   ; gdi+ may now be shutdown on exiting the program
   oGdip := ""
}

F12::ExitApp
