﻿; gdi+ ahk tutorial 1 written by tic (Tariq Porter)
; Requires Gdip.ahk either in your Lib folder as standard library or using #Include
;
; Tutorial to draw a single ellipse and rectangle to the screen

#SingleInstance Force
;#NoEnv
;SetBatchLines -1

; Uncomment if Gdip.ahk is not in your standard library
#Include ../Gdip_All.ahk

; Start gdi+
If !(pToken := Gdip_Startup())
{
    MsgBox "Gdiplus failed to start. Please ensure you have gdiplus on your system"
    ExitApp
}
OnExit("ExitFunc")

; Set the width and height we want as our drawing area, to draw everything in. This will be the dimensions of our bitmap
w := 800, h := 600

; Create a gdi bitmap with width and height of what we are going to draw into it. This is the entire drawing area for everything
; 创建一个gdi位图，其宽度和高度与我们要绘制的位图相同。 这是所有内容的整个绘制区域
hBitmap := CreateDIBSection(w, h)
; Get a device context compatible with the screen
; 获取与屏幕兼容的设备上下文
hDC := CreateCompatibleDC()
; Select the bitmap into the device context
; 添加图片到设备
oOld := SelectObject(hDC, hBitmap)
; Get a pointer to the graphics of the bitmap, for use with drawing functions
; 获取指向位图图形的指针，以与绘图功能一起使用
pGraphics := Gdip_GraphicsFromHDC(hDC)

; Set the smoothing mode to antialias = 4 to make shapes appear smother (only used for vector drawing and filling)
; 将平滑模式设置为“抗锯齿” = 4，以使形状显得更闷（仅用于矢量绘图和填充）
Gdip_SetSmoothingMode(pGraphics, 4)

;画椭圆
; Create a fully opaque red brush (ARGB = Transparency, red, green, blue) to draw a circle
;创建一个完全不透明的红色画笔（ARGB =透明，红色，绿色，蓝色）以画一个圆
pBrush := Gdip_BrushCreateSolid(0xFFFF0000)
; 使用创建的画笔用椭圆填充位图的图形
; 从坐标 x,y 填充w*h的椭圆
Gdip_FillEllipse(pGraphics, pBrush, 100, 300, 200, 300)
; Delete the brush as it is no longer needed and wastes memory
Gdip_DeleteBrush(pBrush)

pBrush := Gdip_BrushCreateSolid(0x660000FF)
Gdip_FillRectangle(pGraphics, pBrush, 300, 300, 300, 200)
Gdip_DeleteBrush(pBrush)

pPen := Gdip_CreatePen(0xFFFF0000, 3)
Gdip_DrawEllipse(pGraphics, pPen, 100, 50, 200, 300)
Gdip_DeletePen(pPen)

pPen := Gdip_CreatePen(0x660000FF, 10)
Gdip_DrawRectangle(pGraphics, pPen, 250, 80, 300, 200)
Gdip_DeletePen(pPen)

; Create a partially transparent, black brush (ARGB = Transparency, red, green, blue) to draw a rounded rectangle with
pBrush := Gdip_BrushCreateSolid(0x77000000)
; Fill the graphics of the bitmap with a rounded rectangle using the brush created
; Filling the entire graphics - from coordinates (0, 0) the entire width and height
; The last parameter (20) is the radius of the circles used for the rounded corners
Gdip_FillRoundedRectangle(pGraphics, pBrush, 0, 0, 100, 100, 20)
; Delete the brush as it is no longer needed and wastes memory
Gdip_DeleteBrush(pBrush)
; Update the specified window we have created (oGui.hwnd) with a handle to our bitmap (hDC), specifying the x,y,w,h we want it positioned on our screen
; With some simple maths we can place the gui in the centre of our primary monitor horizontally and vertically at the specified heigth and width
;UpdateLayeredWindow(oGui.hwnd, hDC, (A_ScreenWidth-Width)//2, (A_ScreenHeight-Height)//2, Width, Height)

; Create a layered window (+E0x80000 : must be used for UpdateLayeredWindow to work!) that is always on top (+AlwaysOnTop), has no taskbar entry or caption
oGui := GuiCreate("-Caption +E0x80000 +LastFound +AlwaysOnTop +ToolWindow +OwnDialogs")
oGui.Show("NA")
; Update the specified window with a handle to our bitmap (hDC), specifying the x,y,w,h we want it positioned on our screen
; So this will position our gui at (0,0) with the Width and Height specified earlier
UpdateLayeredWindow(oGui.hwnd, hDC, 100, 0, w, h)
OnMessage(0x201, "WM_LBUTTONDOWN")

; Select the object back into the hDC
SelectObject(hDC, oOld)

;清除
DeleteObject(hBitmap)
DeleteDC(hDC)
Gdip_DeleteGraphics(pGraphics)
Return

;#######################################################################

; This function is called every time the user clicks on the gui
; The PostMessage will act on the last found window (this being the gui that launched the subroutine, hence the last parameter not being needed)
WM_LBUTTONDOWN()
{
	PostMessage 0xA1, 2
}

ExitFunc(ExitReason, ExitCode)
{
   global
   ; gdi+ may now be shutdown on exiting the program
   Gdip_Shutdown(pToken)
}

F12::ExitApp
