﻿; gdi+ ahk tutorial 12 written by tic (Tariq Porter)
; Requires Gdip.ahk either in your Lib folder as standard library or using #Include
;
; Tutorial to pixelate a bitmap using machine code

#SingleInstance Force
;#NoEnv
;SetBatchLines -1

; Uncomment if Gdip.ahk is not in your standard library
#Include ../Gdip_All.ahk

; Start gdi+
If !pToken := Gdip_Startup()
{
   MsgBox "Gdiplus failed to start. Please ensure you have gdiplus on your system"
   ExitApp
}
OnExit("ExitFunc")

; Create a layered window that is always on top as usual and get a handle to the window
;AHK v1
;Gui, 1: -Caption +E0x80000 +LastFound +OwnDialogs +Owner +AlwaysOnTop
;Gui, 1: Show, NA
oGui := GuiCreate("-Caption +E0x80000 +LastFound +OwnDialogs +Owner +AlwaysOnTop")
oGui.Show("NA")

; Get a bitmap from the image
If FileExist("bs.jpg")
   pBitmap := Gdip_CreateBitmapFromFile("bs.jpg")
If !pBitmap
{
	MsgBox "Could not load the image 'bs.jpg' "
	ExitApp
}
; Get the width and height of the bitmap we have just created from the file
w := Gdip_GetImageWidth(pBitmap)
h := Gdip_GetImageHeight(pBitmap)

; We also need to create
pBitmapOut := Gdip_CreateBitmap(w, h)

; As normal create a gdi bitmap and get the graphics for it to draw into
hBitMap := CreateDIBSection(w, h)
hDC := CreateCompatibleDC()
oOld := SelectObject(hDC, hBitMap)
pGraphics := Gdip_GraphicsFromHDC(hDC)

; Call WM_LBUTTONDOWN every time the gui is clicked, to allow it to be dragged
OnMessage(0x201, "WM_LBUTTONDOWN")

; Update the window with the hDC so that it has a position and dimension for future calls to not
; have to explicitly pass them
UpdateLayeredWindow(oGui.hwnd, hDC, (A_ScreenWidth-w)//2, (A_ScreenHeight-h)//2, w, h)

; Set a timer to update the gui with our pixelated bitmap
;AHK v1
;SetTimer, Update, 50
SetTimer "Update", 50
return

;#######################################################################

Update()
{
    global
    ; Some simple checks to see if we are increasing or decreasing the pixelation
    ; v is the block size of the pixelation and direction (inc/decreasing)
    if (v <= 1)
    {
        v := 1
        direction := !direction
    }
    else if (v >= 30)
    {
        v := 30
        direction := !direction
    }

    ; Call Gdip_PixelateBitmap with the bitmap we retrieved earlier and the block size of the pixels
    ; The function returns the pixelated bitmap, and doesn't dispose of the original bitmap
    Gdip_PixelateBitmap(pBitmap, pBitmapOut, direction ? ++v : --v)

    ; We can optionally clear the graphics we will be drawing to, but if we know there will be no transparencies then
    ; it doesn't matter
    ;Gdip_GraphicsClear(pGraphics)

    ; We then draw our pixelated bitmap into our graphics and dispose of the pixelated bitmap
    Gdip_DrawImage(pGraphics, pBitmapOut, 0, 0, w, h, 0, 0, w, h)

    ; We can now update our window, and don't need to provide a position or dimensions as we don't want them to change
    UpdateLayeredWindow(oGui.hwnd, hDC)
    return
}

;#######################################################################

; This is called on left click to allow to drag
WM_LBUTTONDOWN()
{
   PostMessage 0xA1, 2
}

;#######################################################################

; On exit, dispose of everything created
Esc::
   ExitApp
return

ExitFunc()
{
   global
   Gdip_DisposeImage(pBitmapOut), Gdip_DisposeImage(pBitmap)
   SelectObject(hDC, oOld)
   DeleteObject(hBitMap)
   DeleteDC(hDC)
   Gdip_DeleteGraphics(pGraphics)
   Gdip_Shutdown(pToken)
}

F12::ExitApp
