﻿; gdi+ ahk tutorial 3 written by tic (Tariq Porter)
; Requires Gdip.ahk either in your Lib folder as standard library or using #Include
;
; Tutorial to take make a gui from an existing image on disk
; For the example we will use png as it can handle transparencies. The image will also be halved in size

#SingleInstance Force
;#NoEnv
;SetBatchLines -1

; Uncomment if Gdip.ahk is not in your standard library
#Include ../Gdip_All.ahk

; Start gdi+
If !pToken := Gdip_Startup()
{
	MsgBox "Gdiplus failed to start. Please ensure you have gdiplus on your system"
	ExitApp
}
OnExit("ExitFunc")

; Get a bitmap from the image
pBitmap := Gdip_CreateBitmapFromFile("bs.jpg")

; Check to ensure we actually got a bitmap from the file, in case the file was corrupt or some other error occured
If !pBitmap
{
	MsgBox "Could not load 'background.png'"
	ExitApp
}

; Get the width and height of the bitmap we have just created from the file
; This will be the dimensions that the file is
w := Gdip_GetImageWidth(pBitmap)
h := Gdip_GetImageHeight(pBitmap)

; 创建一个gdi位图，其宽度和高度与我们要绘制的位图相同。 这是所有内容的整个绘制区域
; 我们正在以实际图像大小的一半创建此“画布”
; 我们将其减半，因为我们希望图像以一半的尺寸显示在屏幕上的图形用户界面中
hBitMap := CreateDIBSection(w//2, h//2)

hDC := CreateCompatibleDC()
oOld := SelectObject(hDC, hBitMap)
pGraphics := Gdip_GraphicsFromHDC(hDC)

; 我们不需要前面的示例中绘制图像时所需要的SmoothingMode
; 相反，我们必须设置InterpolationMode。 这指定了如何调整文件大小（调整大小的质量）
; 插值模式已设置为HighQualityBicubic = 7
Gdip_SetInterpolationMode(pGraphics, 7)

; DrawImage会将我们从文件中获取的位图绘制到我们创建的位图的图形中
; 我们要绘制整个图像，但要缩小一半
; 因此，坐标是从源位图的（0,0）以及目标位图中获取的
; 指定了源高度和宽度，以及目标宽度和高度（原始的一半）
; Gdip_DrawImage（pGraphics，pBitmap，dx，dy，dw，dh，sx，sy，sw，sh，Matrix）
; d用于目的地，而s用于源。 我们将不再讨论矩阵（这是为了在绘制时更改颜色）
Gdip_DrawImage(pGraphics, pBitmap, 0, 0, w//2, h//2, 0, 0, w, h)

; Create a layered window (+E0x80000 : must be used for UpdateLayeredWindow to work!) that is always on top (+AlwaysOnTop), has no taskbar entry or caption
oGui := GuiCreate("-Caption +E0x80000 +LastFound +AlwaysOnTop +ToolWindow +OwnDialogs")
oGui.Show("NA")

; Update the specified window we have created (oGui.hwnd) with a handle to our bitmap (hDC), specifying the x,y,w,h we want it positioned on our screen
; So this will position our gui at (0,0) with the Width and Height specified earlier (half of the original image)
UpdateLayeredWindow(oGui.hwnd, hDC, 100, 0, w//2, h//2)


; Select the object back into the hDC
SelectObject(hDC, oOld)

DeleteObject(hBitMap)
DeleteDC(hDC)
Gdip_DeleteGraphics(pGraphics)

; The bitmap we made from the image may be deleted
Gdip_DisposeImage(pBitmap)
Return

;#######################################################################

ExitFunc(ExitReason, ExitCode)
{
   global
   ; gdi+ may now be shutdown on exiting the program
   Gdip_Shutdown(pToken)
}

F12::ExitApp
