﻿; gdi+ ahk tutorial 4 written by tic (Tariq Porter)
; Requires Gdip.ahk either in your Lib folder as standard library or using #Include
;
; Tutorial to fill the screen with randomly hatched ellipses

#SingleInstance Force
;#NoEnv
;SetBatchLines -1

; Uncomment if Gdip.ahk is not in your standard library
#Include ../Gdip_All.ahk
#include d:\AA\TC\soft\AutoHotkey\lib\hyaray.ahk

; Start gdi+
If !pToken := Gdip_Startup()
{
	MsgBox "Gdiplus failed to start. Please ensure you have gdiplus on your system"
	ExitApp
}
OnExit("ExitFunc")

; Get the dimensions of the primary monitor
; these funcs are based off MDMF lib and are now included in the Gdip_All library 
MonitorPrimary := GetPrimaryMonitor()
M := GetMonitorInfo(MonitorPrimary)
hyf_objView(m)
WALeft := M.WALeft 
WATop := M.WATop
WARight := M.WARight
WABottom := M.WABottom
WAWidth := M.WARight-M.WALeft
WAHeight := M.WABottom-M.WATop
;msgbox(WALeft . "`n" . WARight . "`n" . WABottom . "`n" . WAWidth . "`n" . WAHeight)

; Create a gdi bitmap with width and height of the work area
hBitMap := CreateDIBSection(WAWidth, WAHeight)
hDC := CreateCompatibleDC()
oOld := SelectObject(hDC, hBitMap)
pGraphics := Gdip_GraphicsFromHDC(hDC)

; Set the smoothing mode to antialias = 4 to make shapes appear smother (only used for vector drawing and filling)
Gdip_SetSmoothingMode(pGraphics, 4)

pBrush := Gdip_BrushCreateSolid(argb:=0xffffffff)
Gdip_FillRectangle(pGraphics, pBrush, 0, 0, A_ScreenWidth, A_ScreenHeight)

; Create a layered window (+E0x80000) that is always on top (+AlwaysOnTop), has no taskbar entry or caption
;AHK v1
;Gui, 1: -Caption +E0x80000 +LastFound +AlwaysOnTop +ToolWindow +OwnDialogs
;Gui, 1: Show, NA
oGui := GuiCreate("-Caption +E0x80000 +LastFound +AlwaysOnTop +ToolWindow +OwnDialogs")
oGui.Show("NA")

UpdateLayeredWindow(oGui.hwnd, hDC, 0, 0, A_ScreenWidth, A_ScreenHeight)

; Set a timer to draw a new ellipse every 200ms
SetTimer "DrawCircle", 200
Return

;#######################################################################

DrawCircle()
{
    global
    ; Get a random colour for the background and foreground of hatch style used to fill the ellipse,
    ; as well as random brush style, x and y coordinates and width/height

    RandBackColour := Random(0.0, 0xffffffff)
    RandForeColour := Random(0.0, 0xffffffff)
    RandBrush := Random(0, 53)
    RandElipseWidth := Random(1, 200)
    RandElipseHeight := Random(1, 200)
    RandElipsexPos := Random(WALeft, WAWidth-RandElipseWidth)
    RandElipseyPos := Random(WATop, WAHeight-RandElipseHeight)

    ; Create the random brush
    pBrush := Gdip_BrushCreateHatch(RandBackColour, RandForeColour, RandBrush)

    ; Fill the graphics of the bitmap with an ellipse using the brush created
    Gdip_FillEllipse(pGraphics, pBrush, RandElipsexPos, RandElipseyPos, RandElipseWidth, RandElipseHeight)

    ; Update the specified window
    UpdateLayeredWindow(oGui.hwnd, hDC, WALeft, WATop, WAWidth, WAHeight)

    ; Delete the brush as it is no longer needed and wastes memory
    Gdip_DeleteBrush(pBrush)
}

;#######################################################################

ExitFunc(ExitReason, ExitCode)
{
   global
   ; Select the object back into the hDC
   SelectObject(hDC, oOld)

   ; Now the bitmap may be deleted
   DeleteObject(hBitMap)

   ; Also the device context related to the bitmap may be deleted
   DeleteDC(hDC)

   ; The graphics may now be deleted
   Gdip_DeleteGraphics(pGraphics)

   ; ...and gdi+ may now be shutdown
   Gdip_Shutdown(pToken)
}
